@@ -13,9 +13,7 @@
  * General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 /* This file is shared between mutter (src/core/meta-xrandr-shared.h)
@@ -30,7 +28,7 @@
 #define META_XRANDR_SHARED_H
 
 typedef enum {
-  META_POWER_SAVE_UNKNOWN = -1,
+  META_POWER_SAVE_UNSUPPORTED = -1,
   META_POWER_SAVE_ON = 0,
   META_POWER_SAVE_STANDBY,
   META_POWER_SAVE_SUSPEND,
