@@ -44,11 +44,11 @@
 #include "meta-wayland-private.h"
 #include "meta-xwayland-private.h"
 #include "meta-wayland-stage.h"
-#include "meta-surface-actor.h"
 #include "meta-wayland-seat.h"
 #include "meta-wayland-keyboard.h"
 #include "meta-wayland-pointer.h"
 #include "meta-wayland-data-device.h"
+
 #include "meta-cursor-tracker-private.h"
 #include "display-private.h"
 #include "window-private.h"
@@ -58,6 +58,9 @@
 #include "meta-idle-monitor-private.h"
 #include "monitor-private.h"
 
+#include "meta-surface-actor.h"
+#include "meta-surface-actor-wayland.h"
+
 typedef enum
 {
   META_WAYLAND_SUBSURFACE_PLACEMENT_ABOVE,
@@ -79,7 +82,6 @@ surface_handle_buffer_destroy (struct wl_listener *listener, void *data)
   wl_resource_post_error (surface->resource, WL_DISPLAY_ERROR_INVALID_OBJECT,
                           "Destroyed buffer while it was attached to the surface");
   surface->buffer = NULL;
-  wl_list_remove (&surface->buffer_destroy_listener.link);
 }
 
 static void
@@ -114,8 +116,8 @@ surface_process_damage (MetaWaylandSurface *surface,
     {
       cairo_rectangle_int_t rect;
       cairo_region_get_rectangle (region, i, &rect);
-      meta_surface_actor_damage_area (surface->surface_actor,
-                                      rect.x, rect.y, rect.width, rect.height);
+      meta_surface_actor_process_damage (surface->surface_actor,
+                                         rect.x, rect.y, rect.width, rect.height);
     }
 }
 
@@ -259,9 +261,6 @@ ensure_buffer_texture (MetaWaylandBuffer *buffer)
   CoglError *catch_error = NULL;
   CoglTexture *texture;
 
-  if (!buffer)
-    return;
-
   texture = COGL_TEXTURE (cogl_wayland_texture_2d_new_from_buffer (ctx,
                                                                    buffer->resource,
                                                                    &catch_error));
@@ -292,10 +291,10 @@ actor_surface_commit (MetaWaylandSurface             *surface,
   MetaSurfaceActor *surface_actor = surface->surface_actor;
   MetaWaylandBuffer *buffer = pending->buffer;
 
-  if (buffer_changed)
+  if (buffer_changed && buffer)
     {
       ensure_buffer_texture (buffer);
-      meta_surface_actor_attach_wayland_buffer (surface_actor, buffer);
+      meta_surface_actor_wayland_set_buffer (META_SURFACE_ACTOR_WAYLAND (surface->surface_actor), buffer);
     }
 
   surface_process_damage (surface, pending->damage);
@@ -571,23 +570,16 @@ const struct wl_surface_interface meta_wayland_surface_interface = {
   meta_wayland_surface_set_buffer_scale
 };
 
-static void
-unparent_actor (MetaWaylandSurface *surface)
+void
+meta_wayland_surface_make_toplevel (MetaWaylandSurface *surface)
 {
-  ClutterActor *parent_actor;
-
-  parent_actor = clutter_actor_get_parent (CLUTTER_ACTOR (surface->surface_actor));
-  clutter_actor_remove_child (parent_actor, CLUTTER_ACTOR (surface->surface_actor));
+  clutter_actor_set_reactive (CLUTTER_ACTOR (surface->surface_actor), TRUE);
 }
 
 void
 meta_wayland_surface_window_unmanaged (MetaWaylandSurface *surface)
 {
-  /* The window is being unmanaged. Unparent our surface actor
-   * before the window actor is destroyed, as we need to hold
-   * onto it... */
-  unparent_actor (surface);
-
+  clutter_actor_set_reactive (CLUTTER_ACTOR (surface->surface_actor), FALSE);
   surface->window = NULL;
 }
 
@@ -645,29 +637,18 @@ meta_wayland_surface_create (MetaWaylandCompositor *compositor,
   wl_resource_set_implementation (surface->resource, &meta_wayland_surface_interface, surface, wl_surface_destructor);
 
   surface->buffer_destroy_listener.notify = surface_handle_buffer_destroy;
-  surface->surface_actor = g_object_ref_sink (meta_surface_actor_new (surface));
-  clutter_actor_set_reactive (CLUTTER_ACTOR (surface->surface_actor), TRUE);
+  surface->surface_actor = g_object_ref_sink (meta_surface_actor_wayland_new (surface));
 
   double_buffered_state_init (&surface->pending);
-
   return surface;
 }
 
 static void
 destroy_surface_extension (MetaWaylandSurfaceExtension *extension)
 {
-  wl_list_remove (&extension->surface_destroy_listener.link);
   extension->resource = NULL;
 }
 
-static void
-extension_handle_surface_destroy (struct wl_listener *listener,
-				  void *data)
-{
-  MetaWaylandSurfaceExtension *extension = wl_container_of (listener, extension, surface_destroy_listener);
-  wl_resource_destroy (extension->resource);
-}
-
 static int
 get_resource_version (struct wl_resource *master_resource,
                       int                 max_version)
@@ -677,23 +658,22 @@ get_resource_version (struct wl_resource *master_resource,
 
 static gboolean
 create_surface_extension (MetaWaylandSurfaceExtension *extension,
-                          struct wl_client            *client,
-                          struct wl_resource          *master_resource,
-                          struct wl_resource          *surface_resource,
-                          guint32                      id,
                           int                          max_version,
                           const struct wl_interface   *interface,
                           const void                  *implementation,
-                          wl_resource_destroy_func_t   destructor)
+                          wl_resource_destroy_func_t   destructor,
+                          MetaWaylandSurface          *surface,
+                          struct wl_resource          *master_resource,
+                          guint32                      id)
 {
+  struct wl_client *client;
+
   if (extension->resource != NULL)
     return FALSE;
 
+  client = wl_resource_get_client (surface->resource);
   extension->resource = wl_resource_create (client, interface, get_resource_version (master_resource, max_version), id);
-  wl_resource_set_implementation (extension->resource, implementation, extension, destructor);
-
-  extension->surface_destroy_listener.notify = extension_handle_surface_destroy;
-  wl_resource_add_destroy_listener (surface_resource, &extension->surface_destroy_listener);
+  wl_resource_set_implementation (extension->resource, implementation, surface, destructor);
 
   return TRUE;
 }
@@ -720,11 +700,10 @@ xdg_shell_pong (struct wl_client *client,
 static void
 xdg_surface_destructor (struct wl_resource *resource)
 {
-  MetaWaylandSurfaceExtension *xdg_surface = wl_resource_get_user_data (resource);
-  MetaWaylandSurface *surface = wl_container_of (xdg_surface, surface, xdg_surface);
+  MetaWaylandSurface *surface = wl_resource_get_user_data (resource);
 
   destroy_window (surface);
-  destroy_surface_extension (xdg_surface);
+  destroy_surface_extension (&surface->xdg_surface);
 }
 
 static void
@@ -739,8 +718,7 @@ xdg_surface_set_transient_for (struct wl_client *client,
                                struct wl_resource *resource,
                                struct wl_resource *parent_resource)
 {
-  MetaWaylandSurfaceExtension *xdg_surface = wl_resource_get_user_data (resource);
-  MetaWaylandSurface *surface = wl_container_of (xdg_surface, surface, xdg_surface);
+  MetaWaylandSurface *surface = wl_resource_get_user_data (resource);
   MetaWindow *transient_for = NULL;
 
   if (parent_resource)
@@ -760,8 +738,7 @@ xdg_surface_set_margin (struct wl_client *client,
                         int32_t top_margin,
                         int32_t bottom_margin)
 {
-  MetaWaylandSurfaceExtension *xdg_surface = wl_resource_get_user_data (resource);
-  MetaWaylandSurface *surface = wl_container_of (xdg_surface, surface, xdg_surface);
+  MetaWaylandSurface *surface = wl_resource_get_user_data (resource);
 
   surface->pending.frame_extents_changed = TRUE;
   surface->pending.frame_extents.left = left_margin;
@@ -775,8 +752,7 @@ xdg_surface_set_title (struct wl_client *client,
                        struct wl_resource *resource,
                        const char *title)
 {
-  MetaWaylandSurfaceExtension *xdg_surface = wl_resource_get_user_data (resource);
-  MetaWaylandSurface *surface = wl_container_of (xdg_surface, surface, xdg_surface);
+  MetaWaylandSurface *surface = wl_resource_get_user_data (resource);
 
   meta_window_set_title (surface->window, title);
 }
@@ -786,8 +762,7 @@ xdg_surface_set_app_id (struct wl_client *client,
                         struct wl_resource *resource,
                         const char *app_id)
 {
-  MetaWaylandSurfaceExtension *xdg_surface = wl_resource_get_user_data (resource);
-  MetaWaylandSurface *surface = wl_container_of (xdg_surface, surface, xdg_surface);
+  MetaWaylandSurface *surface = wl_resource_get_user_data (resource);
 
   meta_window_set_wm_class (surface->window, app_id, app_id);
 }
@@ -822,8 +797,7 @@ xdg_surface_move (struct wl_client *client,
                   guint32 serial)
 {
   MetaWaylandSeat *seat = wl_resource_get_user_data (seat_resource);
-  MetaWaylandSurfaceExtension *xdg_surface = wl_resource_get_user_data (resource);
-  MetaWaylandSurface *surface = wl_container_of (xdg_surface, surface, xdg_surface);
+  MetaWaylandSurface *surface = wl_resource_get_user_data (resource);
 
   if (seat->pointer.button_count == 0 ||
       seat->pointer.grab_serial != serial ||
@@ -834,7 +808,7 @@ xdg_surface_move (struct wl_client *client,
 }
 
 static MetaGrabOp
-grab_op_for_edge (int edge)
+grab_op_for_xdg_surface_resize_edge (int edge)
 {
   switch (edge)
     {
@@ -868,15 +842,14 @@ xdg_surface_resize (struct wl_client *client,
                     guint32 edges)
 {
   MetaWaylandSeat *seat = wl_resource_get_user_data (seat_resource);
-  MetaWaylandSurfaceExtension *xdg_surface = wl_resource_get_user_data (resource);
-  MetaWaylandSurface *surface = wl_container_of (xdg_surface, surface, xdg_surface);
+  MetaWaylandSurface *surface = wl_resource_get_user_data (resource);
 
   if (seat->pointer.button_count == 0 ||
       seat->pointer.grab_serial != serial ||
       seat->pointer.focus_surface != surface)
     return;
 
-  begin_grab_op_on_surface (surface, seat, grab_op_for_edge (edges));
+  begin_grab_op_on_surface (surface, seat, grab_op_for_xdg_surface_resize_edge (edges));
 }
 
 static void
@@ -891,8 +864,7 @@ static void
 xdg_surface_set_fullscreen (struct wl_client *client,
                             struct wl_resource *resource)
 {
-  MetaWaylandSurfaceExtension *xdg_surface = wl_resource_get_user_data (resource);
-  MetaWaylandSurface *surface = wl_container_of (xdg_surface, surface, xdg_surface);
+  MetaWaylandSurface *surface = wl_resource_get_user_data (resource);
 
   surface->pending.fullscreen.changed = TRUE;
   surface->pending.fullscreen.value = TRUE;
@@ -913,8 +885,7 @@ static void
 xdg_surface_set_maximized (struct wl_client *client,
                            struct wl_resource *resource)
 {
-  MetaWaylandSurfaceExtension *xdg_surface = wl_resource_get_user_data (resource);
-  MetaWaylandSurface *surface = wl_container_of (xdg_surface, surface, xdg_surface);
+  MetaWaylandSurface *surface = wl_resource_get_user_data (resource);
 
   surface->pending.maximized.changed = TRUE;
   surface->pending.maximized.value = TRUE;
@@ -924,8 +895,7 @@ static void
 xdg_surface_unset_maximized (struct wl_client *client,
                              struct wl_resource *resource)
 {
-  MetaWaylandSurfaceExtension *xdg_surface = wl_resource_get_user_data (resource);
-  MetaWaylandSurface *surface = wl_container_of (xdg_surface, surface, xdg_surface);
+  MetaWaylandSurface *surface = wl_resource_get_user_data (resource);
 
   surface->pending.maximized.changed = TRUE;
   surface->pending.maximized.value = FALSE;
@@ -935,8 +905,7 @@ static void
 xdg_surface_set_minimized (struct wl_client *client,
                            struct wl_resource *resource)
 {
-  MetaWaylandSurfaceExtension *xdg_surface = wl_resource_get_user_data (resource);
-  MetaWaylandSurface *surface = wl_container_of (xdg_surface, surface, xdg_surface);
+  MetaWaylandSurface *surface = wl_resource_get_user_data (resource);
 
   meta_window_minimize (surface->window);
 }
@@ -965,11 +934,12 @@ xdg_shell_get_xdg_surface (struct wl_client *client,
 {
   MetaWaylandSurface *surface = wl_resource_get_user_data (surface_resource);
 
-  if (!create_surface_extension (&surface->xdg_surface, client, surface_resource, resource, id,
+  if (!create_surface_extension (&surface->xdg_surface,
                                  META_XDG_SURFACE_VERSION,
                                  &xdg_surface_interface,
                                  &meta_wayland_xdg_surface_interface,
-                                 xdg_surface_destructor))
+                                 xdg_surface_destructor,
+                                 surface, resource, id))
     {
       wl_resource_post_error (surface_resource,
                               WL_DISPLAY_ERROR_INVALID_OBJECT,
@@ -977,17 +947,17 @@ xdg_shell_get_xdg_surface (struct wl_client *client,
       return;
     }
 
+  meta_wayland_surface_make_toplevel (surface);
   surface->window = meta_window_wayland_new (meta_get_display (), surface);
 }
 
 static void
 xdg_popup_destructor (struct wl_resource *resource)
 {
-  MetaWaylandSurfaceExtension *xdg_popup = wl_resource_get_user_data (resource);
-  MetaWaylandSurface *surface = wl_container_of (xdg_popup, surface, xdg_popup);
+  MetaWaylandSurface *surface = wl_resource_get_user_data (resource);
 
   destroy_window (surface);
-  destroy_surface_extension (xdg_popup);
+  destroy_surface_extension (&surface->xdg_popup);
 }
 
 static void
@@ -1020,11 +990,12 @@ xdg_shell_get_xdg_popup (struct wl_client *client,
   if (parent_surf == NULL || parent_surf->window == NULL)
     return;
 
-  if (!create_surface_extension (&surface->xdg_popup, client, surface_resource, resource, id,
+  if (!create_surface_extension (&surface->xdg_popup,
                                  META_XDG_POPUP_VERSION,
                                  &xdg_popup_interface,
                                  &meta_wayland_xdg_popup_interface,
-                                 xdg_popup_destructor))
+                                 xdg_popup_destructor,
+                                 surface, resource, id))
     {
       wl_resource_post_error (surface_resource,
                               WL_DISPLAY_ERROR_INVALID_OBJECT,
@@ -1032,6 +1003,7 @@ xdg_shell_get_xdg_popup (struct wl_client *client,
       return;
     }
 
+  meta_wayland_surface_make_toplevel (surface);
   surface->window = meta_window_wayland_new (meta_get_display (), surface);
   surface->window->rect.x = parent_surf->window->rect.x + x;
   surface->window->rect.y = parent_surf->window->rect.y + y;
@@ -1106,13 +1078,264 @@ bind_xdg_shell (struct wl_client *client,
   wl_client_add_destroy_listener (client, &xdg_shell->client_destroy_listener);
 }
 
+static void
+wl_shell_surface_destructor (struct wl_resource *resource)
+{
+  MetaWaylandSurface *surface = wl_resource_get_user_data (resource);
+
+  destroy_surface_extension (&surface->wl_shell_surface);
+}
+
+static void
+wl_shell_surface_pong (struct wl_client *client,
+                       struct wl_resource *resource,
+                       uint32_t serial)
+{
+  MetaDisplay *display = meta_get_display ();
+
+  meta_display_pong_for_serial (display, serial);
+}
+
+static void
+wl_shell_surface_move (struct wl_client *client,
+                       struct wl_resource *resource,
+                       struct wl_resource *seat_resource,
+                       uint32_t serial)
+{
+  MetaWaylandSeat *seat = wl_resource_get_user_data (seat_resource);
+  MetaWaylandSurface *surface = wl_resource_get_user_data (resource);
+
+  if (seat->pointer.button_count == 0 ||
+      seat->pointer.grab_serial != serial ||
+      seat->pointer.focus_surface != surface)
+    return;
+
+  begin_grab_op_on_surface (surface, seat, META_GRAB_OP_MOVING);
+}
+
+static MetaGrabOp
+grab_op_for_wl_shell_surface_resize_edge (int edge)
+{
+  switch (edge)
+    {
+    case WL_SHELL_SURFACE_RESIZE_TOP_LEFT:
+      return META_GRAB_OP_RESIZING_NW;
+    case WL_SHELL_SURFACE_RESIZE_TOP:
+      return META_GRAB_OP_RESIZING_N;
+    case WL_SHELL_SURFACE_RESIZE_TOP_RIGHT:
+      return META_GRAB_OP_RESIZING_NE;
+    case WL_SHELL_SURFACE_RESIZE_RIGHT:
+      return META_GRAB_OP_RESIZING_E;
+    case WL_SHELL_SURFACE_RESIZE_BOTTOM_RIGHT:
+      return META_GRAB_OP_RESIZING_SE;
+    case WL_SHELL_SURFACE_RESIZE_BOTTOM:
+      return META_GRAB_OP_RESIZING_S;
+    case WL_SHELL_SURFACE_RESIZE_BOTTOM_LEFT:
+      return META_GRAB_OP_RESIZING_SW;
+    case WL_SHELL_SURFACE_RESIZE_LEFT:
+      return META_GRAB_OP_RESIZING_W;
+    default:
+      g_warning ("invalid edge: %d", edge);
+      return META_GRAB_OP_NONE;
+    }
+}
+
+static void
+wl_shell_surface_resize (struct wl_client *client,
+                         struct wl_resource *resource,
+                         struct wl_resource *seat_resource,
+                         uint32_t serial,
+                         uint32_t edges)
+{
+  MetaWaylandSeat *seat = wl_resource_get_user_data (seat_resource);
+  MetaWaylandSurface *surface = wl_resource_get_user_data (resource);
+
+  if (seat->pointer.button_count == 0 ||
+      seat->pointer.grab_serial != serial ||
+      seat->pointer.focus_surface != surface)
+    return;
+
+  begin_grab_op_on_surface (surface, seat, grab_op_for_wl_shell_surface_resize_edge (edges));
+}
+
+typedef enum {
+  SURFACE_STATE_TOPLEVEL,
+  SURFACE_STATE_FULLSCREEN,
+  SURFACE_STATE_MAXIMIZED,
+} SurfaceState;
+
+static void
+wl_shell_surface_set_state (MetaWaylandSurface *surface,
+                            SurfaceState        state)
+{
+  if (state == SURFACE_STATE_FULLSCREEN)
+    meta_window_make_fullscreen (surface->window);
+  else
+    meta_window_unmake_fullscreen (surface->window);
+
+  if (state == SURFACE_STATE_MAXIMIZED)
+    meta_window_maximize (surface->window, META_MAXIMIZE_VERTICAL | META_MAXIMIZE_HORIZONTAL);
+  else
+    meta_window_unmaximize (surface->window, META_MAXIMIZE_VERTICAL | META_MAXIMIZE_HORIZONTAL);
+}
+
+static void
+wl_shell_surface_set_toplevel (struct wl_client *client,
+                               struct wl_resource *resource)
+{
+  MetaWaylandSurface *surface = wl_resource_get_user_data (resource);
+
+  wl_shell_surface_set_state (surface, SURFACE_STATE_TOPLEVEL);
+}
+
+static void
+wl_shell_surface_set_transient (struct wl_client *client,
+                                struct wl_resource *resource,
+                                struct wl_resource *parent_resource,
+                                int32_t x,
+                                int32_t y,
+                                uint32_t flags)
+{
+  MetaWaylandSurface *parent_surf = wl_resource_get_user_data (parent_resource);
+  MetaWaylandSurface *surface = wl_resource_get_user_data (resource);
+
+  wl_shell_surface_set_state (surface, SURFACE_STATE_TOPLEVEL);
+
+  meta_window_set_transient_for (surface->window, parent_surf->window);
+  meta_window_move (surface->window, FALSE,
+                    parent_surf->window->rect.x + x,
+                    parent_surf->window->rect.y + y);
+  surface->window->placed = TRUE;
+}
+
+static void
+wl_shell_surface_set_fullscreen (struct wl_client *client,
+                                 struct wl_resource *resource,
+                                 uint32_t method,
+                                 uint32_t framerate,
+                                 struct wl_resource *output)
+{
+  MetaWaylandSurface *surface = wl_resource_get_user_data (resource);
+
+  wl_shell_surface_set_state (surface, SURFACE_STATE_FULLSCREEN);
+}
+
+static void
+wl_shell_surface_set_popup (struct wl_client *client,
+                            struct wl_resource *resource,
+                            struct wl_resource *seat_resource,
+                            uint32_t serial,
+                            struct wl_resource *parent_resource,
+                            int32_t x,
+                            int32_t y,
+                            uint32_t flags)
+{
+  MetaWaylandSurface *surface = wl_resource_get_user_data (resource);
+  MetaWaylandSurface *parent_surf = wl_resource_get_user_data (parent_resource);
+  MetaWaylandSeat *seat = wl_resource_get_user_data (seat_resource);
+
+  wl_shell_surface_set_state (surface, SURFACE_STATE_TOPLEVEL);
+
+  meta_window_set_transient_for (surface->window, parent_surf->window);
+  meta_window_move (surface->window, FALSE,
+                    parent_surf->window->rect.x + x,
+                    parent_surf->window->rect.y + y);
+  surface->window->placed = TRUE;
+
+  meta_wayland_pointer_start_popup_grab (&seat->pointer, surface);
+}
+
+static void
+wl_shell_surface_set_maximized (struct wl_client *client,
+                                struct wl_resource *resource,
+                                struct wl_resource *output)
+{
+  MetaWaylandSurface *surface = wl_resource_get_user_data (resource);
+
+  wl_shell_surface_set_state (surface, SURFACE_STATE_MAXIMIZED);
+}
+
+static void
+wl_shell_surface_set_title (struct wl_client *client,
+                            struct wl_resource *resource,
+                            const char *title)
+{
+  MetaWaylandSurface *surface = wl_resource_get_user_data (resource);
+
+  meta_window_set_title (surface->window, title);
+}
+
+static void
+wl_shell_surface_set_class (struct wl_client *client,
+                            struct wl_resource *resource,
+                            const char *class_)
+{
+  MetaWaylandSurface *surface = wl_resource_get_user_data (resource);
+
+  meta_window_set_wm_class (surface->window, class_, class_);
+}
+
+static const struct wl_shell_surface_interface meta_wayland_wl_shell_surface_interface = {
+  wl_shell_surface_pong,
+  wl_shell_surface_move,
+  wl_shell_surface_resize,
+  wl_shell_surface_set_toplevel,
+  wl_shell_surface_set_transient,
+  wl_shell_surface_set_fullscreen,
+  wl_shell_surface_set_popup,
+  wl_shell_surface_set_maximized,
+  wl_shell_surface_set_title,
+  wl_shell_surface_set_class,
+};
+
+static void
+wl_shell_get_shell_surface (struct wl_client *client,
+                            struct wl_resource *resource,
+                            uint32_t id,
+                            struct wl_resource *surface_resource)
+{
+  MetaWaylandSurface *surface = wl_resource_get_user_data (surface_resource);
+
+  if (!create_surface_extension (&surface->wl_shell_surface,
+                                 META_WL_SHELL_SURFACE_VERSION,
+                                 &wl_shell_surface_interface,
+                                 &meta_wayland_wl_shell_surface_interface,
+                                 wl_shell_surface_destructor,
+                                 surface, resource, id))
+    {
+      wl_resource_post_error (surface_resource,
+                              WL_DISPLAY_ERROR_INVALID_OBJECT,
+                              "wl_shell::get_shell_surface already requested");
+      return;
+    }
+
+  meta_wayland_surface_make_toplevel (surface);
+  surface->window = meta_window_wayland_new (meta_get_display (), surface);
+}
+
+static const struct wl_shell_interface meta_wayland_wl_shell_interface = {
+  wl_shell_get_shell_surface,
+};
+
+static void
+bind_wl_shell (struct wl_client *client,
+               void             *data,
+               uint32_t          version,
+               uint32_t          id)
+{
+  struct wl_resource *resource;
+
+  resource = wl_resource_create (client, &wl_shell_interface,
+                                 MIN (META_WL_SHELL_VERSION, version), id);
+  wl_resource_set_implementation (resource, &meta_wayland_wl_shell_interface, data, NULL);
+}
+
 static void
 gtk_surface_destructor (struct wl_resource *resource)
 {
-  MetaWaylandSurfaceExtension *gtk_surface = wl_resource_get_user_data (resource);
-  MetaWaylandSurface *surface = wl_container_of (gtk_surface, surface, gtk_surface);
+  MetaWaylandSurface *surface = wl_resource_get_user_data (resource);
 
-  destroy_surface_extension (gtk_surface);
+  destroy_surface_extension (&surface->gtk_surface);
 }
 
 static void
@@ -1125,8 +1348,7 @@ set_dbus_properties (struct wl_client   *client,
 		     const char         *application_object_path,
 		     const char         *unique_bus_name)
 {
-  MetaWaylandSurfaceExtension *gtk_surface = wl_resource_get_user_data (resource);
-  MetaWaylandSurface *surface = wl_container_of (gtk_surface, surface, gtk_surface);
+  MetaWaylandSurface *surface = wl_resource_get_user_data (resource);
 
   /* Broken client, let it die instead of us */
   if (!surface->window)
@@ -1156,11 +1378,12 @@ get_gtk_surface (struct wl_client *client,
 {
   MetaWaylandSurface *surface = wl_resource_get_user_data (surface_resource);
 
-  if (!create_surface_extension (&surface->gtk_surface, client, surface_resource, resource, id,
+  if (!create_surface_extension (&surface->gtk_surface,
                                  META_GTK_SURFACE_VERSION,
                                  &gtk_surface_interface,
                                  &meta_wayland_gtk_surface_interface,
-                                 gtk_surface_destructor))
+                                 gtk_surface_destructor,
+                                 surface, resource, id))
     {
       wl_resource_post_error (surface_resource,
                               WL_DISPLAY_ERROR_INVALID_OBJECT,
@@ -1246,11 +1469,18 @@ subsurface_parent_surface_committed (MetaWaylandSurface *surface)
   double_buffered_state_reset (pending_surface_state);
 }
 
+static void
+unparent_actor (MetaWaylandSurface *surface)
+{
+  ClutterActor *parent_actor;
+  parent_actor = clutter_actor_get_parent (CLUTTER_ACTOR (surface->surface_actor));
+  clutter_actor_remove_child (parent_actor, CLUTTER_ACTOR (surface->surface_actor));
+}
+
 static void
 wl_subsurface_destructor (struct wl_resource *resource)
 {
-  MetaWaylandSurfaceExtension *subsurface = wl_resource_get_user_data (resource);
-  MetaWaylandSurface *surface = wl_container_of (subsurface, surface, subsurface);
+  MetaWaylandSurface *surface = wl_resource_get_user_data (resource);
 
   if (surface->sub.parent)
     {
@@ -1262,7 +1492,7 @@ wl_subsurface_destructor (struct wl_resource *resource)
     }
 
   double_buffered_state_destroy (&surface->sub.pending_surface_state);
-  destroy_surface_extension (subsurface);
+  destroy_surface_extension (&surface->subsurface);
 }
 
 static void
@@ -1278,8 +1508,7 @@ wl_subsurface_set_position (struct wl_client *client,
                             int32_t x,
                             int32_t y)
 {
-  MetaWaylandSurfaceExtension *subsurface = wl_resource_get_user_data (resource);
-  MetaWaylandSurface *surface = wl_container_of (subsurface, surface, subsurface);
+  MetaWaylandSurface *surface = wl_resource_get_user_data (resource);
 
   surface->sub.pending_x = x;
   surface->sub.pending_y = y;
@@ -1329,8 +1558,7 @@ wl_subsurface_place_above (struct wl_client *client,
                            struct wl_resource *resource,
                            struct wl_resource *sibling_resource)
 {
-  MetaWaylandSurfaceExtension *subsurface = wl_resource_get_user_data (resource);
-  MetaWaylandSurface *surface = wl_container_of (subsurface, surface, subsurface);
+  MetaWaylandSurface *surface = wl_resource_get_user_data (resource);
   MetaWaylandSurface *sibling = wl_resource_get_user_data (sibling_resource);
 
   if (!is_valid_sibling (surface, sibling))
@@ -1352,8 +1580,7 @@ wl_subsurface_place_below (struct wl_client *client,
                            struct wl_resource *resource,
                            struct wl_resource *sibling_resource)
 {
-  MetaWaylandSurfaceExtension *subsurface = wl_resource_get_user_data (resource);
-  MetaWaylandSurface *surface = wl_container_of (subsurface, surface, subsurface);
+  MetaWaylandSurface *surface = wl_resource_get_user_data (resource);
   MetaWaylandSurface *sibling = wl_resource_get_user_data (sibling_resource);
 
   if (!is_valid_sibling (surface, sibling))
@@ -1374,10 +1601,7 @@ static void
 wl_subsurface_set_sync (struct wl_client *client,
                         struct wl_resource *resource)
 {
-  MetaWaylandSurfaceExtension *subsurface =
-    wl_resource_get_user_data (resource);
-  MetaWaylandSurface *surface =
-    wl_container_of (subsurface, surface, subsurface);
+  MetaWaylandSurface *surface = wl_resource_get_user_data (resource);
 
   surface->sub.synchronous = TRUE;
 }
@@ -1386,10 +1610,7 @@ static void
 wl_subsurface_set_desync (struct wl_client *client,
                           struct wl_resource *resource)
 {
-  MetaWaylandSurfaceExtension *subsurface =
-    wl_resource_get_user_data (resource);
-  MetaWaylandSurface *surface =
-    wl_container_of (subsurface, surface, subsurface);
+  MetaWaylandSurface *surface = wl_resource_get_user_data (resource);
 
   if (surface->sub.synchronous)
     subsurface_parent_surface_committed (surface);
@@ -1435,11 +1656,12 @@ wl_subcompositor_get_subsurface (struct wl_client *client,
   MetaWaylandSurface *surface = wl_resource_get_user_data (surface_resource);
   MetaWaylandSurface *parent = wl_resource_get_user_data (parent_resource);
 
-  if (!create_surface_extension (&surface->subsurface, client, surface_resource, resource, id,
+  if (!create_surface_extension (&surface->subsurface,
                                  META_GTK_SURFACE_VERSION,
                                  &wl_subsurface_interface,
                                  &meta_wayland_subsurface_interface,
-                                 wl_subsurface_destructor))
+                                 wl_subsurface_destructor,
+                                 surface, resource, id))
     {
       wl_resource_post_error (surface_resource,
                               WL_DISPLAY_ERROR_INVALID_OBJECT,
@@ -1485,6 +1707,11 @@ meta_wayland_init_shell (MetaWaylandCompositor *compositor)
 			compositor, bind_xdg_shell) == NULL)
     g_error ("Failed to register a global xdg-shell object");
 
+  if (wl_global_create (compositor->wayland_display,
+			&wl_shell_interface, 1,
+			compositor, bind_wl_shell) == NULL)
+    g_error ("Failed to register a global wl-shell object");
+
   if (wl_global_create (compositor->wayland_display,
 			&gtk_shell_interface,
 			META_GTK_SHELL_VERSION,
@@ -1506,6 +1733,9 @@ meta_wayland_surface_configure_notify (MetaWaylandSurface *surface,
   if (surface->xdg_surface.resource)
     xdg_surface_send_configure (surface->xdg_surface.resource,
                                 new_width, new_height);
+  else if (surface->wl_shell_surface.resource)
+    wl_shell_surface_send_configure (surface->wl_shell_surface.resource,
+                                     0, new_width, new_height);
 }
 
 void
@@ -1526,16 +1756,23 @@ void
 meta_wayland_surface_ping (MetaWaylandSurface *surface,
                            guint32             serial)
 {
-  struct wl_client *client = wl_resource_get_client (surface->resource);
-  struct wl_resource *xdg_shell = get_xdg_shell_for_client (client);
-
-  if (xdg_shell == NULL)
+  if (surface->xdg_surface.resource)
     {
-      g_warning ("Trying to ping a surface without an xdg_shell bound. How does this happen?");
-      return;
-    }
+      struct wl_client *client = wl_resource_get_client (surface->resource);
+      struct wl_resource *xdg_shell = get_xdg_shell_for_client (client);
 
-  xdg_shell_send_ping (xdg_shell, serial);
+      if (xdg_shell == NULL)
+        {
+          g_warning ("Trying to ping a surface without an xdg_shell bound. How does this happen?");
+          return;
+        }
+
+      xdg_shell_send_ping (xdg_shell, serial);
+    }
+  else if (surface->wl_shell_surface.resource)
+    {
+      wl_shell_surface_send_ping (surface->wl_shell_surface.resource, serial);
+    }
 }
 
 void
@@ -1544,3 +1781,16 @@ meta_wayland_surface_delete (MetaWaylandSurface *surface)
   if (surface->xdg_surface.resource)
     xdg_surface_send_delete (surface->xdg_surface.resource);
 }
+
+void
+meta_wayland_surface_popup_done (MetaWaylandSurface *surface)
+{
+  struct wl_client *client = wl_resource_get_client (surface->resource);
+  struct wl_display *display = wl_client_get_display (client);
+  uint32_t serial = wl_display_next_serial (display);
+
+  if (surface->xdg_popup.resource)
+    xdg_popup_send_popup_done (surface->xdg_popup.resource, serial);
+  else if (surface->wl_shell_surface.resource)
+    wl_shell_surface_send_popup_done (surface->wl_shell_surface.resource);
+}
