@@ -47,10 +47,12 @@ xserver_set_window_id (struct wl_client *client,
   if (!window)
     return;
 
+  meta_wayland_surface_make_toplevel (surface);
+
   surface->window = window;
   window->surface = surface;
 
-  meta_window_set_surface_mapped (window, TRUE);
+  meta_compositor_window_surface_changed (display->compositor, window);
 }
 
 static const struct xserver_interface xserver_implementation = {
